#!/usr/bin/env python

# Use python to work with SATIRE data for PMIP4-CMIP6
#   https://pmip4.lsce.ipsl.fr/doku.php/data:solar_satire
#
# J-Y Peterschmitt - LSCE - 08/2016
#
# Usage: python solar_txt2nc input_full_path output_dir
#
# Note: the following was used in tcsh  in order to process easily the 4 data files
#
# > foreach fbz2 ( `ls /home/motifdb/PMIP4/design/PMIP4_Solar_Jungclaus/files/*bz2` )
# foreach? echo Processing $fbz2
# foreach? echo
# foreach? python solar_txt2nc.py $fbz2 /home/scratch01/jypeter
# foreach? echo
# foreach? end

import bz2, sys
from os import path
import numpy as np

# input_dir = '/home/motifdb/PMIP4/design/PMIP4_Solar_Jungclaus/files/'
# input_file_name = 'SSI_14C_cycle_yearly_cmip_v20160613_fc.txt.bz2'
# input_full_path = '%s/%s' % (input_dir, input_file_name)
# output_dir = '/home/scratch01/jypeter'

input_full_path = sys.argv[1]
print '\nProcessing', input_full_path

input_dir, input_file_name = path.split(input_full_path)

output_dir = sys.argv[2]

nb_header_lines = 12

# Get directly the data from the bz2 compressed file
file_in = bz2.BZ2File(input_full_path)

# Print the comments at the beginning of the file
print '\nData file header:'
for skip_header in range(nb_header_lines):
  hdr = file_in.readline().strip()
  print 'HEADER =>', hdr

# Get the wavelength data (on a single line) and store it in a numpy
# array
wl_str = file_in.readline()
wl = np.array(map(float, wl_str.strip().split()), dtype=np.float32)

# Get the wavelength bins data (on a single line) and store it in a
# numpy array
wl_bin_str = file_in.readline()
wl_bin = np.array(map(float, wl_bin_str.strip().split()), dtype=np.float32)

# Get the years (on a single line) and store them in a numpy array
year_str = file_in.readline()
year = np.array(map(float, year_str.strip().split()), dtype=np.float32)

# The rest of the data is the ssi data, with one line for each time
# steps! Read it in an array
ssi = np.loadtxt(file_in, dtype=np.float32)

file_in.close()

print '\nSize of the SSI matrix =', ssi.shape

# Compute the TSI
tsi = np.dot(ssi, wl_bin)

# WARNING! The following part of the script is used to save the data
# to a NetCDF file, but you need to use the UV-CDAT python
# distribution in order to have access to the cdms2 module
#   http://uv-cdat.llnl.gov

import time, commands, cdms2


# Create the time axis
time_axis = cdms2.createAxis(year)
time_axis.designateTime()
time_axis.units = 'years since 0000-01-01'
time_axis.calendar = 'proleptic_gregorian'
time_axis.note = 'A Gregorian calendar extended to dates before 1582-10-15. That is, a year is a leap year if either (i) it is divisible by 4 but not by 100 or (ii) it is divisible by 400'
time_axis.id = 'time'
                     
# Create the wavelength axis
wl_axis = cdms2.createAxis(wl)
wl_axis.designateLevel()
wl_axis.units = 'nm'
wl_axis.id = 'wavelength'

# Associate SSI and the axes
ssi_masked = np.ma.array(ssi)
ssi_var = cdms2.createVariable(ssi_masked, axes=(time_axis, wl_axis))
ssi_var.id = 'ssi'
ssi_var.units = 'W m-2 nm-1'

# Associate the wavelength bin and the wavelength axis
wl_bin_masked = np.ma.array(wl_bin)
wl_bin_var = cdms2.createVariable(wl_bin_masked, axes=(wl_axis,))
wl_bin_var.units = 'nm'
wl_bin_var.id = 'wavelength_bin'

# Make sure we will create NetCDF3 data
cdms2.setNetcdfShuffleFlag(0)
cdms2.setNetcdfDeflateFlag(0)
cdms2.setNetcdfDeflateLevelFlag(0)

# Create the NetCDF output file, and add some basic information about
# the data provenance
input_md5sum = commands.getoutput('md5sum %s' % (input_full_path,))
input_md5sum = input_md5sum.split()[0]
output_full_path = path.join(output_dir, input_file_name.split('.')[0]+'.nc')

f = cdms2.open(output_full_path, 'w')
f.history = '''File created by J-Y Peterschmitt @ LSCE on %s
from file %s
that had md5sum = %s
More information about the data on PMIP3 wiki:
  https://pmip4.lsce.ipsl.fr/doku.php/data:solar_satire
WARNING! Compare the value of the md5 checksum of this file to the one on the
web site to be sure you have the latest version of the data''' % (time.asctime(), input_file_name, input_md5sum)
f.write(wl_bin_var)
f.write(ssi_var)
f.close()

print '\nFinished creating', output_full_path

# The end
